@@ -6,7 +6,7 @@ import (
 
 	"github.com/zyedidia/micro/v2/internal/buffer"
 	"github.com/zyedidia/micro/v2/internal/display"
-	"github.com/zyedidia/tcell"
+	"github.com/zyedidia/tcell/v2"
 )
 
 type RawPane struct {
@@ -36,8 +36,8 @@ func (h *RawPane) HandleEvent(event tcell.Event) {
 
 	h.Buf.Insert(h.Cursor.Loc, reflect.TypeOf(event).String()[7:])
 
-	switch e := event.(type) {
-	case *tcell.EventKey:
+	e, err := ConstructEvent(event)
+	if err == nil {
 		h.Buf.Insert(h.Cursor.Loc, fmt.Sprintf(": %s", e.Name()))
 	}
 
